package jp.gr.javacons.industry.seminar.tempmonitor;

import javax.comm.*;
import java.io.*;
import java.util.*;
import java.text.NumberFormat;

public class DarwinDataCollector implements DataCollector {

    protected CommPortIdentifier portID;
    protected SerialPort port;
    protected BufferedReader reader;
    protected PrintWriter writer;

    // {Ȃݒt@CȂǂɋLqׂ
    protected String portName = "COM1";
    protected int baudrate = 9600;
    protected int databits = SerialPort.DATABITS_8;
    protected int stopbits = SerialPort.STOPBITS_1;
    protected int parity = SerialPort.PARITY_EVEN;
    protected int flowcontrol = SerialPort.FLOWCONTROL_NONE;

    protected int channelSize = 4; // `l
    protected String type = "TC,K,"; // ͂̃^Cv łK^Md΂gp
    protected int min = -200;      // ŏl
    protected int max = 1370;      // ől

    protected NumberFormat formatter;

    public DarwinDataCollector(){
        // PortIdentifier ̐
        portID = getCommPortIdentifier(portName);

	// lo͂̐`pNX
        NumberFormat formatter = NumberFormat.getInstance();
	// o͌ 3ɂ
        formatter.setMinimumIntegerDigits(3);
    }

    protected CommPortIdentifier getCommPortIdentifier(String portname){
        CommPortIdentifier portID = null;
        try{
            // CommPortIdentifier 擾
            portID = CommPortIdentifier.getPortIdentifier(portname);
        }catch(NoSuchPortException ex){
            System.out.println(portname + " can't be found.");
            System.exit(1);
        }

        return portID;
    }

    public void open(){
        try{
            // |[g̃I[v
            port = (SerialPort)portID.open("DarwinDataCollector", 5000);
        }catch(PortInUseException ex){
            // ^CAEg߂ꍇ
            System.out.println("Other application is using " + portName + ".");
            System.exit(1);
        }

        // ʐM̐ݒ
        setSerialPortParameters(port);

        // Reader/Writer ̒o
        writer = getWriter(port);
        reader = getReader(port);

        // DARWIN ̏
        darwinInitialize();

        // DARWIN ̃Wݒ
        for(int i = 0 ; i < channelSize ; i++){
            setRange(i+1, min, max);
        }
    }
    
    public void close(){
        writer.close();
        try{
            reader.close();
        }catch(IOException ex){
            ex.printStackTrace();
        }
        port.close();
    }

    protected void setSerialPortParameters(SerialPort port){
        try {
            // ʐM̐ݒ
            port.setSerialPortParams(baudrate, databits, stopbits, parity);
            port.setFlowControlMode(flowcontrol);
        } catch (UnsupportedCommOperationException ex){
            ex.printStackTrace();
            System.exit(1);
        }
    }

    protected PrintWriter getWriter(SerialPort port){
        PrintWriter writer = null;
        try {
            // o͗p Writer 𐶐
            writer = new PrintWriter(new BufferedWriter(
                     new OutputStreamWriter(port.getOutputStream())));
        } catch (IOException ex){
            ex.printStackTrace();
            System.exit(1);
        }

        return writer;
    }

    protected BufferedReader getReader(SerialPort port){
        BufferedReader reader = null;
        try {
            // ͗p Reader 𐶐
            reader = new BufferedReader(new InputStreamReader(port.getInputStream()));
        } catch (IOException ex){
            ex.printStackTrace();
            System.exit(1);
        }
        
        return reader;
    }

    // DARWIN ̏
    protected void darwinInitialize(){
        // Darwin Reset
        sendCommand("RS0");
        
        // Darwin Initialize
        sendCommand("RC0");
    }

    // DARWIN ̃Wݒ
    protected void setRange(int ch, int minimum, int maximum){

        // W̐ݒɂ SR R}hgp
        //  SRp1,p2,p3,p4,p5
        //        p1: `lԍ p2: ͂̎ p3: ̃^Cv
        //        p4: ŏl p5: ől
        StringBuffer command = new StringBuffer();
        command.append("SR");
        command.append(formatter.format(ch));
        command.append(",");
        command.append(type);
        command.append(String.valueOf(minimum));
        command.append(",");
        command.append(String.valueOf(maximum));

        sendCommand(command.toString());
    }

    public int getChannelSize(){
        return channelSize;
    }

    public int getMinimumValue(){
        return min;
    }

    public int getMaximumValue(){
        return max;
    }

    public Vector getData(){
        // TS R}hgpāA擾f[^̑Is
        //  TSp1
        //          p1: 0 f[^
        //              1 ݒf[^
        sendCommand("TS0");

        // f[^o͑Oɂ̓gK𑗂Kv
        // gK  ESC T
        sendCommand("\u001bT");

        // f[^̎擾
        return requestData();
    }

    // R}h̑M
    protected String sendCommand(String command) {
        writer.println(command);
        writer.flush();

        // R}hɂ͕K߂l
        String result = null;
        try{
            result = reader.readLine();
        }catch(IOException ex){
            ex.printStackTrace();
        }

        return result;
    }

    protected Vector requestData(){
        // f[^̎擾
        //  FMp1,p2,p3
        //         p1: f[^tH[}bgw 0=ASCII, 1=oCi
        //         p2: o͐擪`l  p3: o͏I`l
        writer.println("FM0,001," + formatter.format(channelSize));
        writer.flush();

        String buffer;
        StringTokenizer tokenizer;
        String value;
        Vector results = new Vector();

        // f[^̎M
        //    1s DATEYYMMDD     YY: N MM:  DD: 
        //        2s TIMEhhmmss     hh:  mm:  ss: b
        //        3s S1S2A1A1A2A2A3A3A4A4UUUUUCCC,}DDDDDE-e
        //                S1: f[^Xe[^X  N: m[} E: ُ
        //                S2: f[^  Xy[X: ԃf[^ E: ŏIf[^
        //                A1,A2,A3,A4: A[Xe[^X
        //                U: P 
        //                C: `lԍ
        //                }: f[^̐
        //                DDDDD: f[^̉
        //                E-e: f[^̎w (e̕ɐ)
	//  ȉA`lA3sڂƓtH[}bgŃf[^Mł
	try{
	    buffer = reader.readLine(); // ԏ͎gpȂ 
	    buffer = reader.readLine(); 
	}catch(IOException ex){
	    ex.printStackTrace();
	    System.exit(1);
	}

        for(int i = 0 ; i < channelSize ; i++){
            try{
                buffer = reader.readLine();
                System.out.print("Get : " + buffer);
                tokenizer = new StringTokenizer(buffer, ",");
                tokenizer.nextToken();
                value = tokenizer.nextToken(); // f[^̐؂肾
                results.add(new Double(value));
            }catch(IOException ex){
                ex.printStackTrace();
                System.exit(1);
            }
        }
        return results;
    }
}

